package MakeRedirect::Output::rewrite;
use strict;
use warnings;
use Carp;
use File::Spec;
use parent 'MakeRedirect::Output';
use MakeRedirect::URI::SRC;

sub new {
    my $class = shift;
    local %_ = @_;
    my $monolithic = delete $_{monolithic};
    my $self = $class->SUPER::new(%_);
    $self->{monolithic} = $monolithic || (!$self->{fh} && !$self->{file});
    $self
}

sub getopt {
    my $self = shift;
    $self->SUPER::getopt(
	'm|monolithic' => 'monolithic',
	'o|output' => 'file'
    );
    if (!$self->{fh} && !$self->{file}) {
	$self->{monolithic} = 1;
    }
}

sub open {
    my $self = shift;
    if ($self->{monolithic}) {
	return $self->SUPER::open(@_);
    }
}

sub ruleset {
    my ($self, $host, $rules) = @_;
    my $fh;
    if (!$self->fh) {
	my $filename = File::Spec->catfile($self->{file},
					   ($host || 'GENERIC')) . '.conf';
        CORE::open($fh, '>', $filename)
	    or croak "can't open output file $filename: $!";
	$self->fh($fh);
	$self->need_close($fh);
    }
    
    if (!$self->{monolithic} && $host ne '') {
	$self->print("# Host $host\n");
    }
    foreach my $key (sort { $b cmp $a } keys %{$rules}) {
	my $r = $rules->{$key};
	my $src = new MakeRedirect::URI::SRC($r->src);
	my $dst = $r->dst;
	if (!$src->path) {
	    $src->path('(.*)');
	    $dst->path('$1');
	}
	if ($self->{monolithic} && $host ne '') {
	    $self->printf("RewriteCond %%{HTTP_HOST} =%s [NC]\n", $host);
	}
	if ($src->query) {
	    $self->printf("RewriteCond %%{QUERY_STRING} %s\n", $src->query);
	    $src->unset_query
	}
	$self->printf("RewriteRule \"^%s\" %s [L,R=%d,NE]\n",
		      $src->path, $dst, $self->http_code);
    }
    if ($fh) {
	$self->close
    }
}

1;

=head1 NAME

rewrite - generate mod_rewrite rules

=head1 SYNOPSIS

B<makeredirect>
B<rewrite>
[B<-m>]
[B<--monolithic>]
I<FILE> ...

=head1 DESCRIPTION

Converts a CSV file with source-destination URL pairs to a set of equivalent
B<mod_rewrite> rules.  The output can be a I<monolithic> single set of rewrite
rules or multiple sets of rules scattered among several output files named by
the B<ServerName> they refer to.

In the I<monolithic> output, each line of the source CSV file that contains
a domain name in the column 1 produces a B<RewriteRule> directive preceded by
a B<RewriteCond> that limits its scope to the given domain name only.  For
example, the input line

     http://example.com/login,/

will produce two lines:

     RewriteCond %{HTTP_HOST} =example.com [NC]
     RewriteRule "^/login" / [L,R=301]

whereas the line

     /logout,/exit

produces

     RewriteRule "^/logout" /exit [L,R=301]

The monolithic mode is assumed in the following cases:

=over 4

=item 1. The output file name is not given.

If the B<--output> option is not supplied, the rules are printed to the
standard output.

=item 2. If the B<--monolithic> (B<-m>) option is given.

=back

Otherwise B<rewrite> operates in I<split> mode.  In split mode, all rules
for a specific domain are grouped together and saved in a file named by the
domain name with the C<.conf> suffix.  The argument to the B<--output> (B<-o>)
option supplies the name of the directory where these files will be stored.
The output directory must exist.

=head1 OPTIONS

=over 4

=item B<-m>, B<--monolithic>

Enable monolithic mode.

=item B<-?>

Produce a short help text and exit.

=item B<--help>

Display the manual page and exit.

=item B<--usage>

Display a short command line usage summary and exit.

=back

=head1 SEE ALSO

B<makeredirect>(1),
L<MakeRedirect>.

=cut

